﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.DataImporter.Command.View;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Sum", 1, ColumnGroupState.DoubleColumnSelected | ColumnGroupState.AnySelectedColumnNotSorted,
   "Aggregation", Position = 2, OptionsView = typeof(AggregationCommandView))]
  public class AggregateWithSumCommand : AggregateCommandBase {
    [StorableConstructor]
    protected AggregateWithSumCommand(bool deserializing) : base(deserializing) { }

    public AggregateWithSumCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
    }

    public override string Description {
      get { return "Aggregate with sum function"; }
    }

    protected override IComparable AggregationFunction(ColumnBase col, int startIndex, int endIndex) {
      double value = 0;
      bool allNull = true;
      IComparable val;
      for (int i = startIndex; i < endIndex; i++) {
        val = col.GetValue(i);
        if (val != null) {
          allNull = false;
          value += (double)val;
        }
      }
      return allNull ? (double?)null : value;
    }
  }
}
